#version 330
// world space normal mapping - fragement shader

uniform vec3 cameraPosition;

uniform sampler2D colorTexture;
uniform sampler2D normalTexture;

uniform Material {
	vec4 m_diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	float test;
} material;

#define MAX_LIGHTS 10
uniform int numLights;
uniform struct Light {
   vec4 position;
   vec3 color;
   float attenuation;
   float ambientCE;
   float spotlightAngle;
   vec3 spotlightDirection;
} lights[MAX_LIGHTS];

in vec3 fragPosition;
in vec2 fragTexCoord;
in vec3 lightDirections[MAX_LIGHTS];
in mat3 TBN;

out vec4 finalColor;

vec3 ApplyLight(Light light, vec3 lightDirection, vec3 surfaceColor, vec3 normal, vec3 surfacePos, vec3 surfaceToCamera) {
    vec3 surfaceToLight;
    float attenuation = 1.0;
    if(light.position.w == 0.0) {
        //directional light
        surfaceToLight = normalize(light.position.xyz);
        attenuation = 1.0; //no attenuation for directional lights
    } else {
        //point light
        surfaceToLight = normalize(lightDirection);
        float distanceToLight = length(lightDirection);
        attenuation = 1.0 / (1.0 + light.attenuation * pow(distanceToLight, 2));

        //cone restrictions (affects attenuation)
        float lightToSurfaceAngle = degrees(acos(dot(-surfaceToLight, normalize(light.spotlightDirection))));
        if(lightToSurfaceAngle > light.spotlightAngle){
            attenuation = 0.0;
        }
    }

    //ambient
    vec3 ambient = light.ambientCE * surfaceColor.rgb * light.color;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.color;

    //specular
    //float specularCoefficient = 0.0;
    //if(diffuseCoefficient > 0.0)
    //specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), 80.0);
    //vec3 specular = specularCoefficient * vec3(1,1,1) * light.color;

    //linear color (color before gamma correction)
    //return ambient + attenuation * (diffuse + specular);
	return ambient + attenuation * diffuse;
}

void main() {
	vec3 normal = TBN * normalize(texture2D(normalTexture, fragTexCoord.st).xyz * 2.0 - 1.0);
	vec3 surfaceToCamera = normalize(cameraPosition - fragPosition);
	vec4 surfaceColor = texture2D(colorTexture, fragTexCoord);
    
    //combine color from all the lights
    vec3 linearColor = vec3(0.0);
    for(int i = 0; i < numLights; ++i)
	{
        linearColor += ApplyLight(lights[i], lightDirections[i], surfaceColor.rgb, normal, fragPosition.xyz, surfaceToCamera);
    }
    
    //final color (after gamma correction)
    vec3 gamma = vec3(1.0/2.2);
	finalColor = vec4(pow(linearColor, gamma), surfaceColor.a);
}
